class All(object):
    pass


class View(All):
    pass


class Edit(All):
    pass


class Create(All):
    pass


class Delete(All):
    pass


class Search(All):
    pass


class Aggregate(All):
    pass
